
# Actionable

[![Code Climate](https://codeclimate.com/github/dluxemburg/actionable.png)](https://codeclimate.com/github/dluxemburg/actionable)

<img src="https://semaphoreapp.com/api/v1/projects/d32a2e65068b7b995bd9e9e2f8beb99a64c7f98d/36068/badge.png">

Actionables, jobs to do later. Scheduled, not queued.

## Installation

Add this line to your application's Gemfile:

    gem 'actionable'

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install actionable

## Usage

### Basic Actionables

Include in model:

```ruby
class Book 

  include Mongoid::Document
  include Actionable::Target

  field :current_page

end
```

Schedule actionable:

```ruby
mybook = Book.new
mybook.schedule_actionable(1.hour.from_now,ReadBook,{pages:27})
```

Make sure the job exists too:

```ruby
class ReadBook < Actionable::Job

  def perform
    target.current_page = target.current_page + payload[:pages]
    target.save
  end

end
```

`schedule_actionable` takes three arguments: when to execute the job, the job class and an optional payload hash. Inside your `Actionable::Job`-inheriting class, the model for which the actionable was created (`mybook` above) is available as `target` and the payload hash is available as `payload`.

The actionables themselves are instances of `Actionable::Action` and are available as `actionables` on the model. They can have lots of stuff done to them:

```ruby
# cancel all actionables
mybook.actionables.each(&:cancel)

# reschedule the first one for two hours from now
mybook.actionables.first.reschedule_for(2.hours.from_now)

# update the payload of the last one
mybook.actionables.last.update_attribute(:payload,{pages:32})
```

`Actionable::Actions` are also Mongoid documents.

### Recurring Actionables

Super simple:

```ruby
# read five pages every five minutes...forever
Actionable::Recurrence.create({
  recurrence_string: "*/12 * * * *",
  job_class: ReadBook,
  payload: {pages:5} 
})
```

### Running Worker

Same as a normal Resque worker, just `require 'actionable/tasks'` at the top of you `Rakefile` to patch Resque's worker to poll Mongoid for actionable when Redis is out of jobs.

## Tests

```
$ rake spec
```

## Contributing

1. Fork it
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request
