require "spec_helper"

describe Actionable::MemoryStore::Action do

  it "should be be defined" do
    expect(defined?(Actionable::MemoryStore::Action)).to be_true
  end

end

describe Actionable::MemoryStore do

  let(:payload) {
    {payload: {name: 'daniel', number: 37}}
  }

  it "should not persist data when not saved" do
    j = Actionable::MemoryStore::Action.new(payload)
    expect(Actionable::MemoryStore.count).to eq(0)
  end

  it "should persist data when inserted" do
    j = Actionable::MemoryStore::Action.new(payload).save
    expect(Actionable::MemoryStore.count).to eq(1)
  end

end

describe Actionable::MongoidStore do

  let(:payload) {
    {payload: {name: 'daniel', number: 37}}
  }

  it "should not persist data when not saved" do
    j = Actionable::MongoidStore::Action.new(payload)
    expect(Actionable::MongoidStore.count).to eq(0)
  end

  it "should persist data when inserted" do
    j = Actionable::MongoidStore::Action.new(payload).save
    expect(Actionable::MongoidStore.count).to eq(1)
  end

  context "scheduled for one hour from now" do

    let(:payload){
      {payload: {name: 'daniel', number: 37}, execution_time: 1.hour.from_now }
    }

    before do
      j = Actionable::MongoidStore::Action.new(payload).save
    end

    it "should be available during the correct time window" do
      expect(Actionable::MongoidStore::Action.between(execution_time:[
        30.minutes.from_now,
        90.minutes.from_now
      ]).count).to eq(1)    
    end

    it "should not be available during the incorrect time window" do
      expect(Actionable::MongoidStore::Action.between(execution_time:[
        15.minutes.from_now,
        45.minutes.from_now
      ]).count).to eq(0)    
    end

  end

end