require "spec_helper"

class TestWorker

  attr_accessor :pid

  def spawn
    @pid = Kernel.spawn('bundle exec rake resque:work TERM_CHILD=1 QUEUES=actionable,actionable_sweep')
  end

  def kill
    Process.kill("TERM",@pid)
    # system("killall resque-1.24.1: Waiting for actionable,actionable_sweep")
  end

end

def run_test_worker(id)
  w = TestWorker.new
  w.spawn
  poll_for_finished(id)
  w.kill
end

def poll_for_finished(id)
  until TestModel.find(id).actionables.first.reload.status == :failed
    sleep(1)
  end
end

describe "Failing Actionable::Job" do

  before do
    system("killall 'resque-1.24.1: Waiting for actionable,actionable_sweep'")
  end



  context "scheduled for one minute ago" do


    it "should be executed by a worker manually" do
      Resque.inline = true
      target = TestModel.create({
        color:'blue',
        shape:'square',
        number:3
      })
      target.schedule_actionable(1.minute.ago,TestFailureJob,{number:2})
      id = target.id
      expect(TestFailureJob).to receive(:stubbed_failure_method)
      # expect(Resque::Failure).to receive(:create)
      expect {
        Actionable::Sweep.perform
      }.to raise_error(StandardError)
      expect(TestModel.find(id).number).to_not eq(6)
      expect(TestModel.find(id).name).to_not eq('blue square')
      expect(Actionable::Action.last.status).to eq(:failed)
      expect(Actionable::Action.last.exception_message).to eq('This is my error')
      Resque.inline = false
    end

    it "should be executed by a worker in another processes", :no do
      target = TestModel.create({
        color:'blue',
        shape:'square',
        number:3
      })
      target.schedule_actionable(1.minute.ago,TestFailureJob,{number:2})
      id = target.id
      run_test_worker(id)
      expect(TestModel.find(id).number).to_not eq(6)
      expect(TestModel.find(id).name).to_not eq('blue square')
      expect(Actionable::Action.last.status).to eq(:failed)
      expect(Actionable::Action.last.exception_message).to eq('This is my error')

      error_json = Redis.new.rpop('resque:failed')
      error_hash = JSON.load(error_json)
      expect(error_hash.fetch('error')).to eq("This is my error")
      expect(error_hash.fetch('payload').fetch('args').first.to_s).to eq(Actionable::Action.last.id.to_s)
    end

  end

end
