require "spec_helper"

class TestWorker

  attr_accessor :pid

  def spawn
    @pid = Kernel.spawn('bundle exec rake resque:work TERM_CHILD=1 QUEUES=actionable,actionable_sweep')
  end

  def kill
    system("killall 'resque-1.24.1: Waiting for actionable,actionable_sweep'")
  end

end

def run_test_worker(id)
  w = TestWorker.new
  w.spawn
  poll_for_finished(id)
  w.kill
end

def poll_for_finished(id)
  while TestModel.find(id).reload.number == 3
    sleep(1)
  end
end

describe Actionable::Job do

  before do
    system("killall 'resque-1.24.1: Waiting for actionable,actionable_sweep'")
  end

  context "scheduled for one minute ago" do

    it "should be executed by a worker manually" do
      DOUBLE = double()
      expect(DOUBLE).to receive(:success)

      def TestJob.on_success(s)
        DOUBLE.success
      end

      Resque.inline = true
      target = TestModel.create({
        color:'blue',
        shape:'square',
        number:3
      })
      target.schedule_actionable(1.minute.ago,TestJob,{number:2})
      id = target.id

      Actionable::Sweep.perform
      expect(TestModel.find(id).number).to eq(6)
      expect(TestModel.find(id).name).to eq('blue square')
      expect(Actionable::Action.last.status).to eq(:complete)
      Resque.inline = false
    end

    it "should be executed by a worker in another processes", :no do
      target = TestModel.create({
        color:'blue',
        shape:'square',
        number:3
      })
      target.schedule_actionable(1.minute.ago,TestJob,{number:2})
      id = target.id
      run_test_worker(id)
      expect(TestModel.find(id).number).to eq(6)
      expect(TestModel.find(id).name).to eq('blue square')


    end

  end

end
