require "spec_helper"

describe Actionable::Sweep do

  let(:target) {
    TestModel.create
  }

  context "single jobs" do

    before do
      target.schedule_actionable(1.hour.from_now,TestJob)
      Timecop.travel(2.hours.from_now)
    end

    it "should leave one job in the to do list before being performed" do
      expect(Resque.size('actionable')).to eq(0)
      expect(Actionable::Action.to_do.count).to eq(1)
    end

    it "should move one job from the to do list to the queue after being performed" do
      Actionable::Sweep.perform
      expect(Actionable::Action.to_do.count).to eq(0)
      expect(Resque.size('actionable')).to eq(1)
    end

    it "should enqueue a job that knows the id for its target" do
      Actionable::Sweep.perform
      expect(Resque.pop('actionable')['args'].first).to eq(target.actionables.first.id.to_s)
    end

    it "should update the actionable's status to 'enqueued'" do
      expect(target.actionables.first.status).to eq(:late)
      Actionable::Sweep.perform
      expect(target.actionables.first.status).to eq(:enqueued)
    end

  end

end
